unit ShellCH;
// The copy hook extension is different from the others in that
// it does not require multiple interfaces. It's also less useful
// than the other extensions, because it is so limited. A copy
// hook extension gets called when a *folder* (not file) is
// moved, copied, deleted, or renamed. It can permit or deny
// the operation. But if the folder is moved etc. using a DOS
// box or something other than the shell, the copy hook gets
// no notification.

interface
uses Ole2, Windows, SysUtils, ShellAPI, ShlObj, DelphCom;
const
  CLSID_DelCopyHook : TGUID = (D1:$6A5EF9E2; D2:$DFCE;
    D3:$11CF; D4: ($9A,$34,$44,$45,$53,$54,$00,$00));

type
  IDSCopyHook = class(IUnknown)
  private
    FRefCount     : Integer;
  public
    constructor Create;
    destructor Destroy; override;
    // IUnknown functions
    function QueryInterface(const WantIID: TIID;
      var ReturnedObject): HResult; override;
    function AddRef: LongInt; override;
    function Release: LongInt; override;
    // ICopyHook functions
    function CopyCallback(Wnd:HWND; wFunc:UINT; wFlags:UINT;
      pszSrcFile:LPSTR; dwSrcAttribs:DWORD; pszDestFile:LPSTR;
      dwDestAttribs:DWORD):UINT; virtual; stdcall;
  end;

  ICHClassFactory = Class(IMyClassFactory)
    function CreateInstance(unkOuter: IUnknown; const WantIid: TIID;
      var ReturnedObject): HResult; override;
  end;


implementation
(****** IDSCopyHook ******)
constructor IDSCopyHook.Create;
begin
  Inherited Create;
  FRefCount := 0;
  Inc(DllRefCount);
end;

destructor IDSCopyHook.Destroy;
begin
  Dec(DllRefCount);
  Inherited Destroy;
end;

function IDSCopyHook.QueryInterface(const WantIID: TIID;
  var ReturnedObject): HResult;
VAR P : IUnknown;
begin
  IF IsEqualIID(WantIID, IID_IUnknown) THEN
    P := Self
  ELSE IF IsEqualIID(WantIID, IID_IShellCopyHook) THEN
    P := Self
  ELSE P := nil;
  Pointer(ReturnedObject) := P;
  IF P = NIL THEN Result := E_NOINTERFACE
  ELSE
    begin
      P.AddRef;
      Result := S_OK;
    end;
end;

function IDSCopyHook.AddRef: LongInt;
begin
  Inc(FRefCount);
  Result := FRefCount;
end;

function IDSCopyHook.Release: LongInt;
begin
  Dec(FRefCount);
  Result := FRefCount;
  IF FRefCount = 0 THEN Free;
end;

function IDSCopyHook.CopyCallback(Wnd:HWND;wFunc:UINT; wFlags:UINT;
  pszSrcFile:LPSTR; dwSrcAttribs:DWORD; pszDestFile:LPSTR;
  dwDestAttribs:DWORD):UINT;
VAR buffer : ARRAY[0..2*MAX_PATH + 40] OF Char;
begin
  CASE wFunc OF
    FO_COPY   : StrCopy(buffer, 'Request to copy ');
    FO_MOVE   : StrCopy(buffer, 'Request to move ');
    FO_RENAME : StrCopy(buffer, 'Request to rename ');
    FO_DELETE : StrCopy(buffer, 'Request to delete ');
    ELSE
      Result := ID_YES;
      Exit;
  end;
  StrCat(buffer, pszSrcFile);
  IF wFunc <> FO_DELETE THEN
    begin
      StrCat(buffer, ' to ');
      StrCat(buffer, pszDestFile);
    end;
  StrCat(buffer, '. Allow it?');
  Result := MessageBox(0, buffer, 'Delphi Copy Hook Example',
    MB_ICONQUESTION OR MB_YESNOCANCEL);
end;

(****** ICHClassFactory ******)
function ICHClassFactory.CreateInstance(unkOuter: IUnknown;
  const WantIid: TIID; var ReturnedObject): HResult;
VAR MyShellExtension: IUnknown;
begin
  IF unkOuter <> NIL then
    begin
      Result := CLASS_E_NOAGGREGATION;
      Exit;
    end;
  MyShellExtension := NIL;
  IF IsEqualIID(WantIID, IID_IUnknown) OR
    IsEqualIID(WantIID, IID_IShellCopyHook) THEN
   MyShellExtension := IDSCopyHook.Create;
  Pointer(ReturnedObject) := MyShellExtension;
  IF MyShellExtension = NIL THEN
    begin
      Pointer(ReturnedObject) := NIL;
      Result := E_OUTOFMEMORY;
      Exit;
    end;
  Result := MyShellExtension.QueryInterface(WantIid, ReturnedObject);
  IF Failed(Result) THEN
    MyShellExtension.Free;
end;

end.
