unit shellCM;

interface
uses Ole2, Windows, Classes, SysUtils, ShellAPI, ShlObj, ShellExt,
  DelphCom, DelFiles;
const
  CLSID_DelContextMenu : TGUID = (D1:$6A5EF9E1; D2:$DFCE;
    D3:$11CF; D4: ($9A,$34,$44,$45,$53,$54,$00,$00));
type
  IMyContextMenu = class(ISatelliteUnknown)
  public
    function QueryContextMenu(Menu:HMENU; indexMenu:UINT;
      idCmdFirst:UINT;idCmdLast:UINT; uFlags:UINT):HResult;
      virtual; stdcall;
    function InvokeCommand(lpici: PCMINVOKECOMMANDINFO): HResult;
      virtual; stdcall;
    function GetCommandString(idCmd:UINT; uType:UINT; var pwReserved:
      UINT; pszName:LPSTR; cchMax:UINT):HResult; virtual; stdcall;
  end;

  IDSContextMenu = class(ISEIContainer)
  protected
    FContextMenu  : IMyContextMenu;  // Satellite interface
  public
    constructor Create;
    destructor Destroy; override; 
    function QueryInterface(const WantIID: TIID;
      var ReturnedObject): HResult; override;
  end;

  ICMClassFactory = Class(IMyClassFactory)
    function CreateInstance(unkOuter: IUnknown; const WantIid: TIID;
      var ReturnedObject): HResult; override;
  end;

implementation

(****** IMyContextMenu ******)
const IDOFFSET_DELITEM = 0;

function IMyContextMenu.QueryContextMenu(Menu:HMENU; indexMenu:
  UINT; idCmdFirst:UINT;idCmdLast:UINT; uFlags:UINT):HResult;
begin
  IF ISEIContainer(fContainer).FNumFiles > 1 THEN
    InsertMenu(Menu, IndexMenu, MF_STRING OR MF_BYPOSITION,
      idCmdFirst + IDOFFSET_DELITEM, '&Average magic number...')
  ELSE InsertMenu(Menu, IndexMenu, MF_STRING OR MF_BYPOSITION,
    idCmdFirst + IDOFFSET_DELITEM, '&Magic number...');
  Result := MakeResult(SEVERITY_SUCCESS, 0, IDOFFSET_DELITEM+1);
end;

function IMyContextMenu.InvokeCommand(lpici: PCMINVOKECOMMANDINFO):
  HResult;
VAR
  N, NumBad,
  OneCount,
  TotalCount : Integer;
  Messg      : String;
begin
  Result := E_FAIL;
  IF HiWord(Integer(lpici.lpVerb)) <> 0 THEN Exit;
  IF LoWord(Integer(lpici.lpVerb)) > IDOFFSET_DELITEM THEN
    begin
      result := E_INVALIDARG;
      exit;
    end;
  TotalCount := 0;
  NumBad := 0;
  WITH ISEIContainer(FContainer) DO
    begin
      FOR N := 0 TO FInitFiles.Count-1 DO
        begin
          OneCount := GetDelFileNumber(FInitFiles[N]);
          IF OneCount < 0 THEN Inc(NumBad)
          ELSE Inc(TotalCount, OneCount);
        end;
      IF FInitFiles.Count > 1 THEN
        begin
          IF NumBad = 0 THEN
            Messg := Format('The %d selected files have an average '+
              'magic number of %d', [FInitFiles.Count,
                TotalCount DIV (FInitFiles.Count)])
          ELSE IF NumBad = FInitFiles.Count THEN
            Messg := Format('The %d selected files all have a bad '+
              'magic number', [NumBad])
          ELSE
            Messg := Format('The %d selected files have an average '+
              'magic number of %d (excluding %d with a bad magic '+
              'number)',
              [FInitFiles.Count, TotalCount DIV
              (FInitFiles.Count-NumBad), NumBad]);
        end
      ELSE
        begin
          IF NumBad > 0 THEN
            Messg := Format('DelFile %s has a bad magic number',
              [FInitFiles[0]])
          ELSE
            Messg := Format('DelFile %s has magic number %d',
              [FInitFiles[0], TotalCount])
        end;
    end;
  MessageBox(0, PChar(Messg), 'Context Menu Hook', MB_OK OR
    MB_ICONINFORMATION);
  Result := NOERROR;
end;

function IMyContextMenu.GetCommandString(idCmd:UINT; uType:UINT;
  var pwReserved: UINT; pszName:LPSTR; cchMax:UINT):HResult;
begin
  Result := NOERROR;
  IF idCmd > IDOFFSET_DELITEM then Result := E_INVALIDARG
  ELSE IF ISEIContainer(fContainer).FNumFiles > 1 THEN
    CASE uType OF
      GCS_VERB     : StrLCopy(pszname, 'ReportAvg', cchMax);
      GCS_HELPTEXT : StrLCopy(pszName, 'Report average '+
        ' magic number of selected files', cchMax);
    end
  ELSE
    CASE uType OF
      GCS_VERB     : StrLCopy(pszname, 'ReportMagic', cchMax);
      GCS_HELPTEXT : StrLCopy(pszName, 'Report magic number '+
        'of selected file', cchMax);
    end;
end;

(****** IDSContextMenu ******)
constructor IDSContextMenu.Create;
begin
  Inherited Create;
  FContextMenu  := IMyContextMenu.Create(Self);
end;

destructor IDSContextMenu.Destroy;
begin
  FContextMenu. Free;
  Inherited Destroy;
end;

function IDSContextMenu.QueryInterface(const WantIID: TIID;
  var ReturnedObject): HResult;
VAR P : IUnknown;
begin
  Result := Inherited QueryInterface(WantIID, ReturnedObject);
  IF Result = S_OK THEN Exit
  ELSE IF IsEqualIID(WantIID, IID_IContextMenu) THEN
    P := FContextMenu
  ELSE P := nil;
  Pointer(ReturnedObject) := P;
  IF P = NIL THEN Result := E_NOINTERFACE
  ELSE
    begin
      P.AddRef;
      Result := S_OK;
    end;
end;

(****** ICMClassFactory ******)
function ICMClassFactory.CreateInstance(unkOuter: IUnknown;
  const WantIid: TIID; var ReturnedObject): HResult;
VAR MyShellExtension: IUnknown;
begin
  IF unkOuter <> NIL then
    begin
      Result := CLASS_E_NOAGGREGATION;
      Exit;
    end;
  MyShellExtension := NIL;
  IF IsEqualIID(WantIID, IID_IUnknown) OR
     IsEqualIID(WantIID, IID_IShellExtInit) OR
     IsEqualIID(WantIID, IID_IContextMenu) THEN
    MyShellExtension := IDSContextMenu.Create;
  Pointer(ReturnedObject) := MyShellExtension;
  IF MyShellExtension = NIL THEN
    begin
      Pointer(ReturnedObject) := NIL;
      Result := E_OUTOFMEMORY;
      Exit;
    end;
  Result := MyShellExtension.QueryInterface(WantIid, ReturnedObject);
  IF Failed(Result) THEN
    MyShellExtension.Free;
end;

end.
