unit shelldd;

interface
uses Ole2, Windows, Classes, SysUtils, ShellAPI, ShlObj, ShellExt,
  DelphCom, DelFiles;
const
  CLSID_DelDragDrop : TGUID = (D1:$6A5EF9E5; D2:$DFCE;
    D3:$11CF; D4: ($9A,$34,$44,$45,$53,$54,$00,$00));
type
  IMyDragDrop = class(ISatelliteUnknown)
  public
    function QueryContextMenu(Menu:HMENU; indexMenu:UINT;
      idCmdFirst:UINT;idCmdLast:UINT; uFlags:UINT):HResult;
      virtual; stdcall;
    function InvokeCommand(lpici: PCMINVOKECOMMANDINFO): HResult;
      virtual; stdcall;
    function GetCommandString(idCmd:UINT; uType:UINT; var pwReserved:
      UINT; pszName:LPSTR; cchMax:UINT):HResult; virtual; stdcall;
  end;

  IDSDragDrop = class(ISEIContainer)
  protected
    FDragDrop  : IMyDragDrop;  // Satellite interface
  public
    constructor Create;
    destructor Destroy; override;
    function QueryInterface(const WantIID: TIID;
      var ReturnedObject): HResult; override;
  end;

  IDDClassFactory = Class(IMyClassFactory)
    function CreateInstance(unkOuter: IUnknown; const WantIid: TIID;
      var ReturnedObject): HResult; override;
  end;

implementation

(****** IMyDragDrop ******)
const IDOFFSET_DELITEM = 0;

function IMyDragDrop.QueryContextMenu(Menu:HMENU; indexMenu:
  UINT; idCmdFirst:UINT;idCmdLast:UINT; uFlags:UINT):HResult;
VAR
  N : Integer;
  AllDelFiles : Boolean;
begin
  AllDelFiles := True;
  WITH ISEIContainer(fContainer) DO
    FOR N := 0 TO FInitFiles.Count-1 DO
      IF UpperCase(ExtractFileExt(fInitFiles[N])) <> '.DEL' THEN
        begin
          AllDelFiles := False;
          Break;
        end;
  IF AllDelFiles THEN
    begin
      InsertMenu(Menu, IndexMenu, MF_STRING OR MF_BYPOSITION,
        idCmdFirst + IDOFFSET_DELITEM, '&Count Files');
      InsertMenu(Menu, IndexMenu+1, MF_SEPARATOR OR MF_BYPOSITION,
        0, '');
      Result := MakeResult(SEVERITY_SUCCESS, 0, IDOFFSET_DELITEM+1);
    end
  else Result := MakeResult(SEVERITY_SUCCESS, 0, 0);
end;

function IMyDragDrop.InvokeCommand(lpici: PCMINVOKECOMMANDINFO):
  HResult;
VAR
  TS     : TSearchRec;
  Rslt, N,
  Count  : Integer;
begin
  Result := E_FAIL;
  IF HiWord(Integer(lpici.lpVerb)) <> 0 THEN Exit;
  IF LoWord(Integer(lpici.lpVerb)) > IDOFFSET_DELITEM THEN
    begin
      result := E_INVALIDARG;
      exit;
    end;
  Count := 0;
  WITH ISEIContainer(fContainer) DO
    begin
      Rslt := FindFirst(FIdPath+'\*.*', 0, TS);
      WHILE Rslt = 0 DO
        begin
          Inc(Count);
          Rslt := FindNext(TS);
        end;
      FindClose(TS);
      FOR N := 0 TO FInitFiles.Count-1 DO
        begin
          IncDelFileNumber(FInitFiles[N], Count);
          SHChangeNotify(SHCNE_UPDATEITEM, SHCNF_PATH OR
            SHCNF_FLUSHNOWAIT, PChar(FInitFiles[N]), NIL);
        end;
    end;
  MessageBeep(0);
  Result := NOERROR;
end;

function IMyDragDrop.GetCommandString(idCmd:UINT; uType:UINT;
  var pwReserved: UINT; pszName:LPSTR; cchMax:UINT):HResult;
begin
  Result := NOERROR;
  IF idCmd > IDOFFSET_DELITEM then Result := E_INVALIDARG
  ELSE
    CASE uType OF
      GCS_VERB     : StrLCopy(pszname, 'CountFiles', cchMax);
      GCS_HELPTEXT : StrLCopy(pszName, 'Count files in dropped-on '+
        'folder and add to magic numbers.', cchMax);
    end;
end;

(****** IDSDragDrop ******)
constructor IDSDragDrop.Create;
begin
  Inherited Create;
  FDragDrop  := IMyDragDrop.Create(Self);
end;

destructor IDSDragDrop.Destroy;
begin
  FDragDrop. Free;
  Inherited Destroy;
end;

function IDSDragDrop.QueryInterface(const WantIID: TIID;
  var ReturnedObject): HResult;
VAR P : IUnknown;
begin
  Result := Inherited QueryInterface(WantIID, ReturnedObject);
  IF Result = S_OK THEN Exit
  ELSE IF IsEqualIID(WantIID, IID_IContextMenu) THEN
    P := FDragDrop
  ELSE P := nil;
  Pointer(ReturnedObject) := P;
  IF P = NIL THEN Result := E_NOINTERFACE
  ELSE
    begin
      P.AddRef;
      Result := S_OK;
    end;
end;

(****** IDDClassFactory ******)
function IDDClassFactory.CreateInstance(unkOuter: IUnknown;
  const WantIid: TIID; var ReturnedObject): HResult;
VAR MyShellExtension: IUnknown;
begin
  IF unkOuter <> NIL then
    begin
      Result := CLASS_E_NOAGGREGATION;
      Exit;
    end;
  MyShellExtension := NIL;
  IF IsEqualIID(WantIID, IID_IUnknown) OR
     IsEqualIID(WantIID, IID_IShellExtInit) OR
     IsEqualIID(WantIID, IID_IContextMenu) THEN
    MyShellExtension := IDSDragDrop.Create;
  Pointer(ReturnedObject) := MyShellExtension;
  IF MyShellExtension = NIL THEN
    begin
      Pointer(ReturnedObject) := NIL;
      Result := E_OUTOFMEMORY;
      Exit;
    end;
  Result := MyShellExtension.QueryInterface(WantIid, ReturnedObject);
  IF Failed(Result) THEN
    MyShellExtension.Free;
end;

end.
