unit shellei;
// The IExtractIcon interface controls which icon is used for
// a given file - it can supply a different icon for each file
// of a given type. This example shows a simple rectangular
// "gauge", with the gauge height depending on the number stored
// in the .DEL file

// If the UseResource directive is defined, this object will
// supply pre-compiled icons. If not, it will create icons on
// the fly.
{..$DEFINE UseResource}
interface
uses Ole2, Windows, Classes, SysUtils, ShellAPI, ShellObj, ShellExt,
  DelphCom, DelFiles{$IFNDEF UseResource}, Graphics{$ENDIF};
const
  CLSID_DelExtractIcon : TGUID = (D1:$6A5EF9E4; D2:$DFCE;
    D3:$11CF; D4: ($9A,$34,$44,$45,$53,$54,$00,$00));
type
  IMyExtractIcon = class(ISatelliteUnknown)
  public
    function GetIconLocation(uFlags:UINT; szIconFile:LPSTR;
      cchMax:UINT; var piIndex:integer; var pwFlags:UINT):
      HResult; virtual; stdcall;
    function Extract(pszFile:LPCSTR; nIconIndex:UINT;
      var phiconLarge:HICON; var phiconSmall:HICON;
      nIconSize:UINT):HResult; virtual; stdcall;
  end;

  IDSExtractIcon = class(IPFContainer)
  protected
    FExtractIcon : IMyExtractIcon; // Satellite interface
  public
    constructor Create;
    destructor Destroy; override;
    function QueryInterface(const WantIID: TIID;
      var ReturnedObject): HResult; override;
  end;

  IEIClassFactory = Class(IMyClassFactory)
    function CreateInstance(unkOuter: IUnknown; const WantIid: TIID;
      var ReturnedObject): HResult; override;
  end;

implementation

(****** IMyExtractIcon ******)
{$IFDEF UseResource}
{$R EI.RES}
function IMyExtractIcon.GetIconLocation(uFlags : UINT; szIconFile :
  LPSTR; cchMax : UINT; var piIndex : integer; var pwFlags:UINT) :
  HResult;
VAR DFN : Integer;
begin
  DFN := GetDelFileNumber(FContainer.FPFFilename);
  CASE DFN OF
    0..44       : piIndex := (DFN DIV 5) + 1;
    45..MaxInt  : piIndex := 9;
    ELSE          piIndex := 0;
  END;
  GetModuleFilename(hInstance, szIconFile, cchMax);
  pwFlags := GIL_PERINSTANCE OR GIL_DONTCACHE;
  Result := S_OK;
end;

function IMyExtractIcon.Extract(pszFile : LPCSTR; nIconIndex : UINT;
  var phiconLarge : HICON; var phiconSmall : HICON; nIconSize :
  UINT) : HResult;
begin
  Result := S_FALSE;
end;

{$ELSE}
function IMyExtractIcon.GetIconLocation(uFlags : UINT; szIconFile :
  LPSTR; cchMax : UINT; var piIndex : integer; var pwFlags:UINT) :
  HResult;
begin
  WITH IPFContainer(fContainer) DO
    piIndex := GetDelFileNumber(FPFFilename);
  pwFlags := GIL_PERINSTANCE OR GIL_DONTCACHE OR GIL_NOTFILENAME;
  Result := NOERROR;
end;

function IMyExtractIcon.Extract(pszFile : LPCSTR; nIconIndex : UINT;
  var phiconLarge : HICON; var phiconSmall : HICON; nIconSize :
  UINT) : HResult;

  function IconGraph(dim, dfNum : Integer): hIcon;
  VAR
    vTB              : Windows.TBitmap;
    PictBmp, MaskBmp : Graphics.TBitmap;
    PictSiz, MaskSiz : Integer;
    NewPict, NewMask : PByteArray;
  begin
    Result := 0;
    PictBmp := TBitmap.Create;
    MaskBmp := TBitmap.Create;
    try
      WITH PictBmp DO
        begin
          Width  := dim;
          Height := dim;
          Canvas.Brush.Color := clBlack;
          Canvas.Pen.Color := clBlack;
          Canvas.FillRect(Rect(0, 0, dim, dim));
          GetObject(Handle, SizeOf(vTB), @vTB);
          PictSiz := (Height * Width * vTB.bmBitsPixel *
            vTB.bmPlanes) DIV 8;
          MaskSiz := (Height * Width) DIV 8;
          Canvas.Pen.Color   := clBlue;
          Canvas.Brush.Color := clWhite;
          Canvas.Rectangle(dim DIV 4, 0, 3*(dim DIV 4), dim);
          IF dfNum < 0 THEN {bad value}
            begin
              Canvas.MoveTo(dim DIV 4, 0);
              Canvas.LineTo(3*(dim DIV 4)-1, dim-1);
              Canvas.MoveTo(dim DIV 4, dim-1);
              Canvas.LineTo(3*(dim DIV 4)-1, 0);
            end
          ELSE IF dfNum >= dim-2 THEN {full}
            begin
              Canvas.Brush.Color := clRed;
              Canvas.FillRect(Rect((dim DIV 4)+1, 1,
                3*(dim DIV 4)-1, dim-1));
            end
          ELSE
            begin
              Canvas.Brush.Color := clRed;
              Canvas.FillRect(Rect((dim DIV 4)+1, dim-1-dfNum,
                3*(dim DIV 4)-1, dim-1));
            end;
        end;
      WITH MaskBmp DO
        begin
          Width := dim;
          Height := dim;
          Canvas.Brush.Color := clWhite;
          Canvas.FillRect(Rect(0, 0, dim, dim));
          Monochrome := True;
          Canvas.Brush.Color := clBlack;
          Canvas.FillRect(Rect(dim DIV 4, 0, 3*(dim DIV 4), dim));
        end;
      GetMem(NewPict, PictSiz);
      GetMem(NewMask, MaskSiz);
      try
        GetBitmapBits(PictBmp.Handle, PictSiz, NewPict);
        GetBitmapBits(MaskBmp.Handle, MaskSiz, NewMask);
        Result := CreateIcon(hInstance, vTB.bmWidth, vTB.bmHeight,
          vTB.bmPlanes, vTB.bmBitsPixel, NewMask, NewPict);
      finally
        FreeMem(NewPict, PictSiz);
        FreeMem(NewMask, MaskSiz);
      end;
    finally
      PictBmp.Free;
      MaskBmp.Free;
    end;
  end;

begin
  phIconLarge := IconGraph(LoWord(nIconSize), nIconIndex);
  phIconSmall := IconGraph(HiWord(nIconSize), nIconIndex);
  IF (phIconLarge <> 0) AND (phIconSmall<>0) THEN
    Result := NOERROR
  ELSE Result := S_FALSE;
end;
{$ENDIF}

(****** IDSExtractIcon ******)
constructor IDSExtractIcon.Create;
begin
  Inherited Create;
  FExtractIcon := IMyExtractIcon.Create(Self);
end;

destructor IDSExtractIcon.Destroy;
begin
  FExtractIcon. Free;
  Inherited Destroy;
end;

function IDSExtractIcon.QueryInterface(const WantIID: TIID;
  var ReturnedObject): HResult;
VAR P : IUnknown;
begin
  Result := Inherited QueryInterface(WantIID, ReturnedObject);
  IF Result = S_OK THEN Exit
  ELSE IF IsEqualIID(WantIID, IID_IExtractIcon) THEN
    P := FExtractIcon
  ELSE P := nil;
  Pointer(ReturnedObject) := P;
  IF P = NIL THEN Result := E_NOINTERFACE
  ELSE
    begin
      P.AddRef;
      Result := S_OK;
    end;
end;

(****** IEIClassFactory ******)
function IEIClassFactory.CreateInstance(unkOuter: IUnknown;
  const WantIid: TIID; var ReturnedObject): HResult;
VAR MyShellExtension: IUnknown;
begin
  IF unkOuter <> NIL then
    begin
      Result := CLASS_E_NOAGGREGATION;
      Exit;
    end;
  MyShellExtension := NIL;
  IF IsEqualIID(WantIID, IID_IUnknown) OR
     IsEqualIID(WantIID, IID_IPersistFile) OR
     IsEqualIID(WantIID, IID_IExtractIcon) THEN
    MyShellExtension := IDSExtractIcon.Create;
  Pointer(ReturnedObject) := MyShellExtension;
  IF MyShellExtension = NIL THEN
    begin
      Pointer(ReturnedObject) := NIL;
      Result := E_OUTOFMEMORY;
      Exit;
    end;
  Result := MyShellExtension.QueryInterface(WantIid, ReturnedObject);
  IF Failed(Result) THEN
    MyShellExtension.Free;
end;

end.
