unit ShellPS;
// The .DEL file property sheet display the "magic number" of
// each selected .DEL file, and permits zeroing out the numbers.

interface
uses Ole2, Windows, Messages, Classes, SysUtils, ShellAPI, ShlObj,
  CommCtrl, ShellExt, DelphCom, DelFiles;
const
  CLSID_DelPropSheet : TGUID = (D1:$6A5EF9E3; D2:$DFCE;
    D3:$11CF; D4: ($9A,$34,$44,$45,$53,$54,$00,$00));
type
  IMyPropSheet = class(ISatelliteUnknown)
  public
    function AddPages(lpfnAddPage : TFNADDPROPSHEETPAGE; lParam:
      LPARAM) : HResult; virtual; stdcall;
    function ReplacePage(uPageID:UINT; lpfnReplaceWith :
      TFNADDPROPSHEETPAGE; lParam: LPARAM) : HResult; virtual; stdcall;
  end;

  IDSPropSheet = class(ISEIContainer)
  protected
    FPropSheet    : IMyPropSheet;    // Satellite interface
  public
    constructor Create;
    destructor Destroy; override;
    function QueryInterface(const WantIID: TIID;
      var ReturnedObject): HResult; override;
  end;

  IPSClassFactory = Class(IMyClassFactory)
    function CreateInstance(unkOuter: IUnknown; const WantIid: TIID;
      var ReturnedObject): HResult; override;
  end;

implementation
(****** IDSPropSheet ******)
constructor IDSPropSheet.Create;
begin
  Inherited Create;
  FPropSheet    := IMyPropSheet.Create(Self);
end;

destructor IDSPropSheet.Destroy;
begin
  FPropSheet.   Free;
  Inherited Destroy;
end;

function IDSPropSheet.QueryInterface(const WantIID: TIID;
  var ReturnedObject): HResult;
VAR P : IUnknown;
begin
  Result := Inherited QueryInterface(WantIID, ReturnedObject);
  IF Result = S_OK THEN Exit
  ELSE IF IsEqualIID(WantIID, IID_IShellPropSheetExt) THEN
    P := FPropSheet
  ELSE P := nil;
  Pointer(ReturnedObject) := P;
  IF P = NIL THEN Result := E_NOINTERFACE
  ELSE
    begin
      P.AddRef;
      Result := S_OK;
    end;
end;

(****** IMyPropSheet ******)
{$R SHEET.RES}
{$I SHEET.INC}
function PropExtCallback(hWndDlg: HWnd; Msg: Integer;
  VAR PPSP: TPropSheetPage): Integer; Far; stdcall;
begin
  CASE Msg OF
    PSPCB_RELEASE : begin
      IF PPSP.lParam <> 0 THEN
        StrDispose(PChar(PPSP.lParam));
    end;
  end;
end;

function PropExtDlgProc(hwndDlg: HWnd; Msg: UINT; wParam: wParam;
  lParam: LPARAM) : Bool; Far; stdcall;
VAR
  N, Num : Integer;
  B : ARRAY[0..Max_PATH] OF Char;
  P : PChar;
begin
  IF Msg = WM_INITDIALOG THEN
    begin
      WITH TStringList.Create DO
        try
          SetText(PChar(PPropSheetPage(lParam)^.lParam));
          Num := 20;
          SendDlgItemMessage(hwndDlg, IDC_ListBox, LB_SETTABSTOPS,
            1, Integer(@Num));
          SetDlgItemText(hwndDlg, IDC_Static, PChar(Format(
            '%d files selected',[Count])));
          FOR N := 0 TO Count-1 DO
            begin
              Num := GetDelFileNumber(Strings[N]);
              IF Num < 0 THEN
                SendDlgItemMessage(hWndDlg, IDC_ListBox, LB_ADDSTRING,
                  0, Integer(PChar('bad'#9+Strings[N])))
              ELSE
                SendDlgItemMessage(hWndDlg, IDC_ListBox, LB_ADDSTRING,
                  0, Integer(PChar(Format('%d'#9'%s',[Num,
                  Strings[N]]))));
            end;
        finally
          Free;
        end;
      StrDispose(PChar(PPropSheetPage(lParam)^.lParam));
      PPropSheetPage(lParam)^.lParam := 0;
      SetWindowLong (hwndDlg, DWL_MSGRESULT, 0);
      Result := TRUE;
    end
  ELSE IF (Msg = WM_COMMAND) AND (LoWord(wParam)=IDC_BUTTON) THEN
    begin
      Num := SendDlgItemMessage(hwndDlg, IDC_ListBox, LB_GETCOUNT,
        0, 0);
      FOR N := 0 TO Num-1 DO
        begin
          SendDlgItemMessage(hwndDlg, IDC_LISTBOX, LB_GETTEXT, N,
            LongInt(@B));
          SendDlgItemMessage(hwndDlg, IDC_LISTBOX, LB_DELETESTRING,
            N, 0);
          P := StrScan(B, #9) + 1;
          SetDelFileNumber(StrPas(P), 0);
          SHChangeNotify(SHCNE_UPDATEITEM, SHCNF_PATH OR
            SHCNF_FLUSHNOWAIT, P, NIL);
          SendDlgItemMessage(hwndDlg, IDC_LISTBOX, LB_INSERTSTRING,
            N, Integer(PChar('0'#9+StrPas(P))));
        end;
    end
  ELSE Result := FALSE;
end;

function IMyPropSheet.AddPages(lpfnAddPage : TFNADDPROPSHEETPAGE;
  lParam : LPARAM) : HResult;
VAR
  TPSP : TPropSheetPage;
  HPSP : HPropSheetPage;
  P    : PChar;
begin
  TPSP.dwSize    := SizeOf(TPSP);
  TPSP.dwFlags   := PSP_USEREFPARENT OR PSP_USETITLE OR
    PSP_USECALLBACK;
  TPSP.hInstance := hInstance;
  TPSP.pszTemplate := MakeIntResource(psDialog);
  TPSP.pszTitle := 'Property Sheet Example';
  TPSP.pfnDlgProc := @PropExtDlgProc;
  TPSP.pfnCallBack := @PropExtCallback;
  TPSP.pcRefParent := @DLLRefCount;
  P := ISEIContainer(FContainer).FInitFiles.GetText;
  TPSP.lParam := Integer(P);
  HPSP := CreatePropertySheetPage(TPSP);
  IF HPSP <> NIL THEN
    begin
      IF NOT lpfnAddPage(HPSP, lParam) THEN
        DestroyPropertySheetPage(HPSP);
    end;
  Result := NOERROR;
end;

function IMyPropSheet.ReplacePage(uPageID:UINT; lpfnReplaceWith :
  TFNADDPROPSHEETPAGE; lParam : LPARAM) : HResult;
begin
  Result := E_NOTIMPL;
end;

(****** IPSClassFactory ******)
function IPSClassFactory.CreateInstance(unkOuter: IUnknown;
  const WantIid: TIID; var ReturnedObject): HResult;
VAR MyShellExtension: IUnknown;
begin
  IF unkOuter <> NIL then
    begin
      Result := CLASS_E_NOAGGREGATION;
      Exit;
    end;
  MyShellExtension := NIL;
  IF IsEqualIID(WantIID, IID_IUnknown) OR
     IsEqualIID(WantIID, IID_IShellExtInit) OR
     IsEqualIID(WantIID, IID_IShellPropSheetExt) THEN
    MyShellExtension := IDSPropSheet.Create;
  Pointer(ReturnedObject) := MyShellExtension;
  IF MyShellExtension = NIL THEN
    begin
      Pointer(ReturnedObject) := NIL;
      Result := E_OUTOFMEMORY;
      Exit;
    end;
  Result := MyShellExtension.QueryInterface(WantIid, ReturnedObject);
  IF Failed(Result) THEN
    MyShellExtension.Free;
end;

end.
