unit FolderScan;

interface

uses
  Classes, Windows;

type
  PExplodeMenuItem = ^TExplodeMenuItem;
  TExplodeMenuItem = record
    IdCmd: Integer;
    Position: Integer;
    Caption: String;
    FileName: String;
  end;

  function NewMenuItem(Menu: HMENU; Path: String; Position, IdCmd: Integer;
               IdCmdOffset: Integer; Caption: String): Boolean;
  function PopulateMenu(FileList: TStringList; IdCmdOffset: Integer): HMENU;
  procedure ErrorMessage(S: String);

var
  ExplodeMenuItems: TList;        // of PExplodeMenuItem objects


implementation

uses
  Version, Shortcut, SysUtils;

const
  MAX_FILES = 30;


function NewMenuItem(Menu: HMENU; Path: String; Position, IdCmd: Integer;
             IdCmdOffset: Integer; Caption: String): Boolean;
var
  emi: PExplodeMenuItem;
begin
//  Result := AppendMenu(Menu, MF_STRING, IdCmd, PChar(Caption));
  Result := InsertMenu(Menu, Position, MF_STRING or MF_BYPOSITION, IdCmd, PChar(Caption));

  emi := New(PExplodeMenuItem);
  FillChar(emi^, SizeOf(emi^), 0);
  emi.IdCmd := IdCmd - IdCmdOffset;
  emi.Position := Position;
  emi.Caption := Caption;
  emi.FileName := Path;
  ExplodeMenuItems.Add(emi);
end;


function PopulateMenu(FileList: TStringList; IdCmdOffset: Integer): HMENU;
var
  I: Integer;
  MenuItemsCount: Integer;
  HPopup: HMENU;
  FileName: String;
  S: String;
  si: TShortcutInfo;
  ShortcutString: String;
begin
  HPopup := 0;
  MenuItemsCount := 0;

  try
    for I := 0 to FileList.Count -1 do
    begin
      // Check for max # files
      if MenuItemsCount >= MAX_FILES then
      begin
        AppendMenu(HPopup, MF_SEPARATOR, 0, nil);
        AppendMenu(HPopup, MF_STRING or MF_DISABLED, 0,
                   PChar('(More than ' + IntToStr(MAX_FILES) + ' files)'));
        Break;
      end;

      FileName := FileList[I];
      ShortcutString := '';

      // Check if file is a shortcut
      if LowerCase(ExtractFileExt(FileName)) = '.lnk' then
        if GetShortcutInfo(FileName, si) then
        begin
          FileName := si.Path;
          ShortcutString := ' (shortcut)';
        end;

      if FileExists(FileName) then     // It might be a directory
      begin
        S := GetBuildInfoString(FileName);
        if S <> '' then
        begin
          // We have a file containing version info
          if MenuItemsCount = 0 then
            HPopup := CreateMenu;      // Create the submenu - we'll need it

          NewMenuItem(HPopup, FileName, MenuItemsCount, IdCmdOffset + MenuItemsCount,
                      IdCmdOffset, ExtractFileName(FileName) + ShortcutString +
                      ' ' + Chr(9) + GetBuildInfoString(FileName));
          Inc(MenuItemsCount);
        end;
      end;
    end;

  except
    on E: Exception do
      ErrorMessage('PopulateMenu: Something bad happened.' + #13 + E.Message);
  end;

  Result := HPopup;
end;


procedure ErrorMessage(S: String);
begin
  MessageBox(0, PChar(S), 'Version Info', MB_OK or MB_ICONERROR);
end;


initialization
  ExplodeMenuItems := TList.Create;
finalization
  ExplodeMenuItems.Free;
end.


