unit shellCM;

interface

uses
  Windows, ShellExt, DelphCom, Ole2, ShlObj;

const
  // The GUID that's found in the verinfo.inf file:
  CLSID_VerInfoContextMenu: TGUID = (D1:$347E1434; D2:$163A;
    D3:$4895; D4:($81,$53,$4B,$A2,$C6,$7A,$DC,$70));

type
  IMyContextMenu = class(ISatelliteUnknown)
  public
    // Be sure to declare the methods in this order!
    function QueryContextMenu(Menu: HMENU; indexMenu,
      idCmdFirst, idCmdLast, uFlags: UINT): HResult; virtual; stdcall;
    function InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult;
      virtual; stdcall;
    function GetCommandString(idCmd, uFlags: UINT; var pwReserved: UINT;
      pszName: LPSTR; cchMax: UINT): HResult; virtual; stdcall;
  end;

  IDSContextMenu = class(ISEIContainer)
  protected
    FContextMenu: IMyContextMenu;  // Satellite interface
  public
    constructor Create;
    destructor Destroy; override;
    function QueryInterface(const WantIID: TIID; var ReturnedObject): HResult; override;
  end;

  ICMClassFactory = class(IMyClassFactory)
    function CreateInstance(unkOuter: IUnknown; const WantIid: TIID;
      var ReturnedObject): HResult; override;
  end;


implementation

uses
  FolderScan, Version, Utility, SysUtils;

const
  IDOFFSET_MENUITEMS = 0;


(****** IMyContextMenu ******)
function IMyContextMenu.QueryContextMenu(Menu: HMENU; indexMenu,
  idCmdFirst, idCmdLast, uFlags: UINT): HResult;
var
  HPopup: HMENU;
  MenuItemsCount: Integer;
  FileName: String;
  S: String;
begin
  Result := S_OK;
  try

    if uFlags = CMF_DEFAULTONLY then
      Exit;

    if ISEIContainer(FContainer).NumFiles < 1 then
      Exit;

    MenuItemsCount := 0;

    if ISEIContainer(FContainer).NumFiles = 1 then
    begin
      FileName := ISEIContainer(FContainer).InitFiles[0];
      S := GetBuildInfoString(FileName);
      if S <> '' then
      begin
        NewMenuItem(Menu, FileName, $FFFFFFFF, idCmdFirst, idCmdFirst,
                    '&Version: ' + GetBuildInfoString(FileName));
        Inc(MenuItemsCount);
      end;
    end

    else if ISEIContainer(FContainer).NumFiles > 1 then
    begin
      ISEIContainer(FContainer).InitFiles.Sort;

      HPopup := PopulateMenu(ISEIContainer(FContainer).InitFiles, idCmdFirst);

      // Get submenu (if any)
      if HPopup <> 0 then
      begin
        InsertMenu(Menu, idCmdLast, MF_STRING or MF_BYPOSITION or MF_POPUP,
          HPopup, PChar('&Version info'));
        MenuItemsCount := GetMenuItemCount(HPopup);
      end;
    end;

  except
    on E: Exception do
    begin
      ErrorMessage('QueryContextMenu: Something bad happened.' + #13 + E.Message);
      Result := S_FALSE;
    end;
  end;

  try
    Result := MakeResult(SEVERITY_SUCCESS, 0, MenuItemsCount+1);
  except
    ErrorMessage('MakeResult failed.');
    Result := S_FALSE;
  end;
end;


function IMyContextMenu.InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult;
var
  idCmd: UINT;
begin
  { If the high-order word of lpici->lpVerb is not NULL, this
    function was called by an application and lpVerb is a command
    that should be activated. Otherwise, the shell has called this
    function, and the low-order word of lpici->lpVerb is the
    identifier of the menu item that the user selected. }
  try
    idCmd := LoWord(UINT(lpici.lpVerb));
    InvokeAction(idCmd);
  except
    on E: Exception do
      ErrorMessage('InvokeCommand: Something bad happened.' + #13 + E.Message);
  end;

  Result := S_OK;
end;

(*
function IMyContextMenu.InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult;
var
  idCmd: UINT;
  fEx: Boolean;
  fUnicode: Boolean;
begin
  { If the high-order word of lpici->lpVerb is not NULL, this
    function was called by an application and lpVerb is a command
    that should be activated. Otherwise, the shell has called this
    function, and the low-order word of lpici->lpVerb is the
    identifier of the menu item that the user selected. }
  try
//    ShowMessage(IntToStr(lpici.cbSize) + ', ' + IntToStr(SizeOf(TCMInvokeCommandInfo)));
    fEx := False;
    fUnicode := False;

    if lpici.cbSize > SizeOf(TCMInvokeCommandInfo) then
    begin
      fEx := True;
      if (lpici.fMask - CMIC_MASK_UNICODE >= 0) then
        fUnicode := True;
    end;

    if (not fUnicode) then
    begin
      ShowMessage('NOT Unicode');
      idCmd := LoWord(UINT(lpici.lpVerb));
    end
    else
    begin
      ShowMessage('Unicode');
      idCmd := LoWord(UINT(lpici.lpVerb));
    end;
//ShowMessage(IntToStr(idCmd));
}
{
    if HiWord(UINT(@lpici.lpVerb)) = 0 then
    begin
ShowMessage('A');
      idCmd := LoWord(UINT(@lpici.lpVerb));
ShowMessage(IntToStr(idCmd));
    end
    else
    begin
ShowMessage('B');
//ShowMessage('Verb: '+lpici.lpVerb);
      idCmd := LoWord(UINT(@lpici.lpVerb));
ShowMessage(IntToStr(idCmd));
    end;
}
    InvokeAction(idCmd);
  except
    on E: Exception do
      ErrorMessage('InvokeCommand: Something bad happened.' + #13 + E.Message);
  end;

  Result := S_OK;
end;
*)

function IMyContextMenu.GetCommandString(idCmd, uFlags: UINT;
  var pwReserved: UINT; pszName: LPSTR; cchMax: UINT): HResult;
const
  HELPTEXT = 'Displays version info of the selected items. Select an item to display its properties.';
  VERB = 'verinfo';
begin
//showmessage('uFlags: '+IntToStr(uFlags));
  Result := S_OK;
  case uFlags of
    GCS_VERBA:
      StrLCopy(pszName, PChar(VERB), cchMax);
    GCS_VERBW:
      StrLCopy(pszName, WideStringAsPChar(VERB), cchMax);
    GCS_HELPTEXTA:
      StrLCopy(pszName, PChar(HELPTEXT), cchMax);
    GCS_HELPTEXTW:
      StrLCopy(pszName, WideStringAsPChar(HELPTEXT), cchMax);
  end;
end;


(****** IDSContextMenu ******)
constructor IDSContextMenu.Create;
begin
  inherited Create;
  FContextMenu := IMyContextMenu.Create(Self);
end;


destructor IDSContextMenu.Destroy;
var
  I: Integer;
begin
  FContextMenu.Free;

  // Dispose items in ExplodeMenuItems list and clear list.
  // The list itself is not destroyed until Windows shuts down (dll unloads).
  for I := ExplodeMenuItems.Count -1 downto 0 do
    Dispose(PExplodeMenuItem(ExplodeMenuItems[I]));
  ExplodeMenuItems.Clear;

  inherited Destroy;
end;


function IDSContextMenu.QueryInterface(const WantIID: TIID;
  var ReturnedObject): HResult;
var
  P: IUnknown;
begin
  Result := inherited QueryInterface(WantIID, ReturnedObject);
  if Result = S_OK then
    Exit
  else if IsEqualIID(WantIID, TGUID(IID_IContextMenu)) then
    P := FContextMenu
  else
    P := nil;
  Pointer(ReturnedObject) := P;
  if P = nil then
    Result := E_NOINTERFACE
  else
  begin
    P.AddRef;
    Result := S_OK;
  end;
end;


(****** ICMClassFactory ******)
function ICMClassFactory.CreateInstance(unkOuter: IUnknown;
  const WantIid: TIID; var ReturnedObject): HResult;
var
  MyShellExtension: IUnknown;
begin
  if unkOuter <> nil then
  begin
    Result := CLASS_E_NOAGGREGATION;
    Exit;
  end;
  MyShellExtension := nil;
  if IsEqualIID(WantIID, IID_IUnknown) or
     IsEqualIID(WantIID, TGUID(IID_IShellExtInit)) or
     IsEqualIID(WantIID, TGUID(IID_IContextMenu)) then
    MyShellExtension := IDSContextMenu.Create;
  Pointer(ReturnedObject) := MyShellExtension;
  if MyShellExtension = nil then
  begin
    Pointer(ReturnedObject) := nil;
    Result := E_OUTOFMEMORY;
    Exit;
  end;
  Result := MyShellExtension.QueryInterface(WantIid, ReturnedObject);
  if Failed(Result) then
    MyShellExtension.Free;
end;

end.

