unit ShellExt;
// The IExtractIcon shell extension relies on the IPersistFile
// interface, and the IContextMenu, IShellPropertySheet, and
// IDragDrop extensions rely on the IShellExtInit interface. This
// unit defines satellite versions of the support interfaces
// IPersistFile and IShellExtInit, along with container object types
// for each. Together, these objects form a middle layer between the
// totally generic satellite and container object types and the
// totally specific final implementations.

interface

uses
  Windows, Classes, Ole2, ShlObj, DelphCom;

type
  IMyShellExtInit = class(ISatelliteUnknown)
  public
    function Initialize(pidlFolder:PItemIDList; lpdobj: IDataObject;
      hKeyProgID: HKEY): HResult; virtual; stdcall;
  end;

  ISEIContainer = class(IContainerUnknown)
  protected
    FShellExtInit: IMyShellExtInit;    // Satellite interface
  public
    NumFiles: Integer;
    InitFiles: TStringList;
    IDPath: String;
    constructor Create;
    destructor Destroy; override;
    function QueryInterface(const WantIID: TIID;
      var ReturnedObject): HResult; override;
  end;
(*
  IMyPersistFile = class(ISatelliteUnknown)
  public
    function GetClassID(var classID: TCLSID): HResult; virtual;
     stdcall;
    function IsDirty: HResult; virtual; stdcall;
    function Load(pszFileName: POleStr; dwMode: Longint): HResult;
      virtual; stdcall;
    function Save(pszFileName: POleStr; fRemember: BOOL): HResult;
      virtual; stdcall;
    function SaveCompleted(pszFileName: POleStr): HResult;
      virtual; stdcall;
    function GetCurFile(var pszFileName: POleStr): HResult;
      virtual; stdcall;
  end;
*)
(*
  IPFContainer = class(IContainerUnknown)
  protected
    FPersistFile: IMyPersistFile;      // Satellite interface
  public
    FPFFilename: String;
    constructor Create;
    destructor Destroy; override;
    function QueryInterface(const WantIID: TIID;
      var ReturnedObject): HResult; override;
  end;
*)
implementation

uses
  SysUtils, ShellApi;

(****** ISEIContainer ******)
constructor ISEIContainer.Create;
begin
  inherited Create;
  FShellExtInit := IMyShellExtInit.Create(Self);
  InitFiles := TStringList.Create;
end;

destructor ISEIContainer.Destroy;
begin
  InitFiles.Free;
  FShellExtInit.Free;
  inherited Destroy;
end;

function ISEIContainer.QueryInterface(const WantIID: TIID;
  var ReturnedObject): HResult;
var
  P: IUnknown;
begin
  Result := inherited QueryInterface(WantIID, ReturnedObject);
  if Result = S_OK then
    Exit
  else if IsEqualIID(WantIID, TGUID(IID_IShellextInit)) then
    P := FShellExtInit
  else
    P := nil;
  Pointer(ReturnedObject) := P;
  if P = nil then
    Result := E_NOINTERFACE
  else
  begin
    P.AddRef;
    Result := S_OK;
  end;
end;

(****** IMyShellExtInit ******)
function IMyShellExtInit.Initialize(pidlFolder: PItemIDList;
  lpdobj: IDataObject; hKeyProgID: HKEY): HResult;
var
  Buffer: array[0..MAX_PATH] of char;
  Medium: TStgMedium;
  fe    : TFormatEtc;
  N     : Integer;
begin
  if lpdobj = nil then
  begin
    Result := E_FAIL;
    Exit;
  end;
  with fe do
  begin
    cfFormat := CF_HDROP;
    ptd      := nil;
    dwAspect := DVASPECT_CONTENT;
    lindex   := -1;
    tymed    := TYMED_HGLOBAL;
  end;
  Result := lpdobj.GetData(fe, medium);
  if Failed(Result) then
    Exit;
  SHGetPathFromIDList(pidlFolder, Buffer);
  with ISEIContainer(FContainer) do
  begin
    IdPath := StrPas(Buffer);
//    FNumFiles := DragQueryFile(medium.hGlobal, 0, nil, 0);
    NumFiles := DragQueryFile(medium.hGlobal, $FFFFFFFF, nil, 0);
    InitFiles.Clear;
    if NumFiles > 0 then
    begin
      for N := 0 to NumFiles -1 do
      begin
        DragQueryFile(medium.hGlobal, N, Buffer, 260);
        InitFiles.Add(StrPas(Buffer));
      end;
      Result := NOERROR;
    end
    else
      Result := E_FAIL;
  end;
  ReleaseStgMedium(medium);
end;

(****** IPFContainer ******)
(*
constructor IPFContainer.Create;
begin
  inherited Create;
  FPFFileName   := '';
  FPersistFile := IMyPersistFile.Create(Self);
end;

destructor IPFContainer.Destroy;
begin
  FPersistFile.Free;
  inherited Destroy;
end;

function IPFContainer.QueryInterface(const WantIID: TIID;
  var ReturnedObject): HResult;
var
  P: IUnknown;
begin
  Result := inherited QueryInterface(WantIID, ReturnedObject);
  if Result = S_OK then Exit
  else if IsEqualIID(WantIID, IID_IPersistFile) then
    P := FPersistFile
  else P := nil;
  Pointer(ReturnedObject) := P;
  if P = nil then Result := E_NOINTERFACE
  else
    begin
      P.AddRef;
      Result := S_OK;
    end;
end;
*)
(****** IMyPersistFile ******)
(*
function IMyPersistFile.GetClassID(var classID: TCLSID): HResult;
begin
  Result := E_FAIL;
end;

function IMyPersistFile.IsDirty: HResult;
begin
  Result := E_FAIL;
end;

function IMyPersistFile.Load(pszFileName: POleStr; dwMode: Longint): HResult;
begin
  IPFContainer(FContainer).FPFFilename :=
    WideCharToString(pszFilename);
  Result := S_OK;
end;

function IMyPersistFile.Save(pszFileName: POleStr; fRemember: BOOL): HResult;
begin
  Result := E_FAIL;
end;

function IMyPersistFile.SaveCompleted(pszFileName: POleStr): HResult;
begin
  Result := E_FAIL;
end;

function IMyPersistFile.GetCurFile(var pszFileName: POleStr): HResult;
begin
  Result := E_FAIL;
end;
*)
end.

